@@ -25,9 +25,9 @@
 #'
 #' # 'left eye' et 'right eye' binaries
 #' bin.left.eye <- bin.from.roi (CT, struct = S, roi.sname = "lefteye",
-#'                               alias = "left eye")
+#'                               alias = "left eye", verbose = FALSE)
 #' bin.right.eye <- bin.from.roi (CT, struct = S, roi.name = "righteye",
-#'                                alias = "right eye")
+#'                                alias = "right eye", verbose = FALSE)
 #' bin.eyes <- bin.sum (bin.left.eye, bin.right.eye, alias = "eyes")
 #'
 #' display.plane (CT, top = bin.eyes, struct = S, roi.sname = "eye",
@@ -37,27 +37,23 @@
 #' @export
 #' @importFrom methods is
 bin.sum <- function (vol1, vol2 , alias = "", description = NULL) {
-  if (!is (vol1, "volume") | !is (vol2, "volume")){
-    warning ("vol1 or vol2 should be volume class objects.")
-    return (NULL)
+  
+  if ((!is (vol1, "volume") & !is.null(vol1)) | (!is (vol2, "volume") & !is.null(vol1))) 
+    stop ("vol1 or vol2 should be volume class objects.")
+  if (!is.null(vol1)){
+    if (vol1$modality!="binary") stop ("vol1 must be of binary modality.")
+    if (is.null(vol1$vol3D.data)) stop ("empty vol1$vol3D.data.")
   }
-  if ((vol1$modality!="binary") | (vol2$modality!="binary")) {
-    warning ("both volumes must be of binary modality.")
-    return (NULL)
-  }
-  if(is.null(vol1$vol3D.data)){
-    warning ("empty vol1$vol3D.data.")
-    return (NULL)
-  }
-  if(is.null(vol2$vol3D.data)){
-    warning ("empty vol2$vol3D.data.")
-    return (NULL)
+  if (!is.null(vol2)){
+    if (vol2$modality!="binary") stop ("vol2 must be of binary modality.")
+    if (is.null(vol2$vol3D.data)) stop ("empty vol2$vol3D.data.")
   }
+  if (is.null(vol1)) return (vol2)
+  if (is.null(vol2)) return (vol1)
+
   #verifier que les volumes ont le même support
-  if (!grid.equal (vol1, vol2)) {
-    warning ("both volumes must share the same grid.")
-    return (NULL)
-  }
+  if (!grid.equal (vol1, vol2)) stop ("both volumes must share the same grid.")
+ 
   if (is.null(description)) description <-  paste (vol1$object.alias, "+", vol2$object.alias)
   Vb <- vol.copy (vol1, alias = alias, modality = "binary",
                   description = description)
