@@ -102,7 +102,7 @@ dicom.viewer <- function(dcm, txt.sep = "\\", txt.length = 100,
       height = height,
       noUiSliderInput(inputId="sI", label=NULL,min=1, max= sum(df_$display)-nbrow,
                       # range=c(1,sum(df_$display)-nbrow),
-                      value=1,  step=1,  update_on="change",
+                      value=1,  step=1,  update_on="end",#behaviour = 'snap',
                       tooltips =FALSE, orientation="vertical",
                       color="#D0D0D0",
                       height = paste0(round((nbrow+1)*fontsize*2.1),"px"),
